package gnu.telnetj;

/**
* <br><br><center><table border="1" width="80%"><hr>
* <strong><a href="http://www.amherst.edu/~tliron/telnetj">telnetj</a></strong>
* <p>
* Copyright (C) 2001 by Tal Liron
* <p>
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2.1
* of the License, or (at your option) any later version.
* <p>
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* <a href="http://www.gnu.org/copyleft/lesser.html">GNU Lesser General Public License</a>
* for more details.
* <p>
* You should have received a copy of the <a href="http://www.gnu.org/copyleft/lesser.html">
* GNU Lesser General Public License</a> along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
* <hr></table></center>
**/

import java.net.*;
import java.io.*;
import java.util.*;

public class Host implements Runnable
{
    //
    // Main
    //

    public static void main( String argv[] ) throws Exception
    {
        ResourceBundle properties = null;
        try
        {
            properties = ResourceBundle.getBundle( "telnetj" );
        }
        catch( MissingResourceException e )
        {
            System.err.println(e);
            System.err.println( "Can't load 'telnetj.properties'." );
            return;
        }

        hostMap = new HashMap();

        // Create hosts
        String key;
        int port;
        String cliClassName;
        Class cliClass;
        for( Enumeration e = properties.getKeys(); e.hasMoreElements(); )
        {
            key = (String) e.nextElement();
            if( key.startsWith( "class." ) )
            {
                port = Integer.parseInt( key.substring( 6 ) );
                cliClassName = properties.getString( key );

                try
                {
                    cliClass = Class.forName( cliClassName );

                    new Host( port, new CommandLineInterfaceFactory( cliClass ) );
                }
                catch( ClassNotFoundException x )
                {
                    System.out.println( "  Could not load class: " + cliClassName );
                }
                catch( NoSuchMethodException x )
                {
                    System.out.println( "  Unsupported class: " + cliClassName );
                }
            }
        }
    }

    public int getPort()
    {
        return port;
    }

    //
    // Construction
    //

    public Host( int port, InterfaceFactory interfaceFactory )
    {
        this.port = port;
        this.interfaceFactory = interfaceFactory;
        this.thread = new Thread(this);

        hostMap.put(new Integer(port), this);

        this.thread.start();
    }

    //public void shutdown()
    //{
    //    int i;
    //    for (i = 0; i < connections.size(); i++)
    //    {
    //        try
    //        {
    //            TelnetConnection t = (TelnetConnection)connections.get(i);
    //            t.close();
    //        }
    //        catch(java.io.IOException e)
    //        {}
    //    }
    //    this.thread.destroy();
    //}

    //
    // Runnable
    //

    public void run()
    {
        ServerSocket serverSocket = null;
        try
        {
            serverSocket = new ServerSocket( port );
        }
        catch( IOException x )
        {
            x.printStackTrace();
            return;
        }

        System.out.println( "telnetj host started on port " + port );

        Socket socket;
        while( true )
        {
            try
            {
                // Accept connections
                socket = serverSocket.accept();

                TelnetConnection t = new TelnetConnection(socket, this);
                interfaceFactory.connect( t );
            }
            catch( Exception x )
            {
                x.printStackTrace();
            }
        }
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    // Private

    private int port;
    private InterfaceFactory interfaceFactory;
    public Thread thread;
    public static HashMap hostMap;
}
